/*
   vt100 display driver for DAMP 
   By Adam Briggs

   This driver was written to display DAMP output on a
   handheld computer running a vt100 emulator attached
   to the serial port of the host computer running DAMP.

   Since completing the driver I have found that purchasing
   the null modem cable that I need to use my handheld for a
   display would cost slightly more than buying a surplus
   20x4 LCD and attaching it to the paralell port (as many
   other people have done).

   If you are a person that happens to have an extra cradle
   or null modem cable for your handheld, however, this little
   bit of scripting may save you the little bit of soldering
   that an LCD would have caused you.

   Don't forget to change the line that initializes the COM_PORT
   constant so that it uses the COM_PORT that your display is
   attached to. If after you do that this driver still does not
   work for you, be sure not to e-mail me because you can count
   on me not caring.

   Have Fun,
   Adam
*/
#include "damplcd.sh"

#define COM1               0
#define COM2               1

#define BIOSCOM_INIT       0
#define BIOSCOM_SEND       1
#define BIOSCOM_RECV       2
#define BIOSCOM_STAT       3

#define BIOSINIT_7BIT      0x02
#define BIOSINIT_8BIT      0x03
#define BIOSINIT_0STOP     0x00
#define BIOSINIT_1STOP     0x04
#define BIOSINIT_NPAR      0x00
#define BIOSINIT_OPAR      0x08
#define BIOSINIT_EPAR      0x18
#define BIOSINIT_110BAUD   0x00
#define BIOSINIT_150BAUD   0x20
#define BIOSINIT_300BAUD   0x40
#define BIOSINIT_600BAUD   0x60
#define BIOSINIT_1200BAUD  0x80
#define BIOSINIT_2400BAUD  0xa0
#define BIOSINIT_4800BAUD  0xc0
#define BIOSINIT_9600BAUD  0xe0

#define VT100_INIT         "c"
#define VT100_HOME         "[H"
#define VT100_CLEAR        "[2J"
#define VT100_CLEARTOEOS   "[J"

const char  VT100_ESCAPE = 27 ; 

const int   COM_PORT = COM1 ; /* Change this to the comport your display is on */
const char  COM_INIT = BIOSINIT_8BIT | BIOSINIT_1STOP | BIOSINIT_NPAR |
                       BIOSINIT_9600BAUD ;

char letters[26][5][7] ;
char numbers[10][5][7] ;
char punctuation[1][5][7] ;

/*
 * I had some difficulties using normal C
 * const array initializations and so the
 * init_font function was born as a workaround.
 */
void init_font()
{
  sprintf(letters[0][0], " AAA  ") ;
  sprintf(letters[0][1], "A   A ") ;
  sprintf(letters[0][2], "AAAAA ") ;
  sprintf(letters[0][3], "A   A ") ;
  sprintf(letters[0][4], "A   A ") ;

  sprintf(letters[1][0], "BBBB  ") ;
  sprintf(letters[1][1], "B   B ") ;
  sprintf(letters[1][2], "BBBB  ") ;
  sprintf(letters[1][3], "B   B ") ;
  sprintf(letters[1][4], "BBBB  ") ;

  sprintf(letters[2][0], " CCC  ") ;
  sprintf(letters[2][1], "C   C ") ;
  sprintf(letters[2][2], "C     ") ;
  sprintf(letters[2][3], "C   C ") ;
  sprintf(letters[2][4], " CCC  ") ;

  sprintf(letters[3][0], "DDDD  ") ;
  sprintf(letters[3][1], "D   D ") ;
  sprintf(letters[3][2], "D   D ") ;
  sprintf(letters[3][3], "D   D ") ;
  sprintf(letters[3][4], "DDDD  ") ;

  sprintf(letters[4][0], "EEEEE ") ;
  sprintf(letters[4][1], "E     ") ;
  sprintf(letters[4][2], "EEE   ") ;
  sprintf(letters[4][3], "E     ") ;
  sprintf(letters[4][4], "EEEEE ") ;

  sprintf(letters[5][0], "FFFFF ") ;
  sprintf(letters[5][1], "F     ") ;
  sprintf(letters[5][2], "FFFF  ") ;
  sprintf(letters[5][3], "F     ") ;
  sprintf(letters[5][4], "F     ") ;

  sprintf(letters[6][0], " GGG  ") ;
  sprintf(letters[6][1], "G   G ") ;
  sprintf(letters[6][2], "G     ") ;
  sprintf(letters[6][3], "G GGG ") ;
  sprintf(letters[6][4], " GGG  ") ;

  sprintf(letters[7][0], "H   H ") ;
  sprintf(letters[7][1], "H   H ") ;
  sprintf(letters[7][2], "HHHHH ") ;
  sprintf(letters[7][3], "H   H ") ;
  sprintf(letters[7][4], "H   H ") ;

  sprintf(letters[8][0], " III  ") ;
  sprintf(letters[8][1], "  I   ") ;
  sprintf(letters[8][2], "  I   ") ;
  sprintf(letters[8][3], "  I   ") ;
  sprintf(letters[8][4], " III  ") ;

  sprintf(letters[9][0], "    J ") ;
  sprintf(letters[9][1], "    J ") ;
  sprintf(letters[9][2], "    J ") ;
  sprintf(letters[9][3], "J   J ") ;
  sprintf(letters[9][4], " JJJ  ") ;

  sprintf(letters[10][0], "K   K ") ;
  sprintf(letters[10][1], "K  K  ") ;
  sprintf(letters[10][2], "KKK   ") ;
  sprintf(letters[10][3], "K  K  ") ;
  sprintf(letters[10][4], "K   K ") ;

  sprintf(letters[11][0], "L     ") ;
  sprintf(letters[11][1], "L     ") ;
  sprintf(letters[11][2], "L     ") ;
  sprintf(letters[11][3], "L     ") ;
  sprintf(letters[11][4], "LLLLL ") ;

  sprintf(letters[12][0], "M   M ") ;
  sprintf(letters[12][1], "MM MM ") ;
  sprintf(letters[12][2], "M M M ") ;
  sprintf(letters[12][3], "M   M ") ;
  sprintf(letters[12][4], "M   M ") ;

  sprintf(letters[13][0], "N   N ") ;
  sprintf(letters[13][1], "NN  N ") ;
  sprintf(letters[13][2], "N N N ") ;
  sprintf(letters[13][3], "N  NN ") ;
  sprintf(letters[13][4], "N   N ") ;

  sprintf(letters[14][0], " OOO  ") ;
  sprintf(letters[14][1], "O   O ") ;
  sprintf(letters[14][2], "O   O ") ;
  sprintf(letters[14][3], "O   O ") ;
  sprintf(letters[14][4], " OOO  ") ;

  sprintf(letters[15][0], "PPPP  ") ;
  sprintf(letters[15][1], "P   P ") ;
  sprintf(letters[15][2], "PPPP  ") ;
  sprintf(letters[15][3], "P     ") ;
  sprintf(letters[15][4], "P     ") ;

  sprintf(letters[16][0], " QQQ  ") ;
  sprintf(letters[16][1], "Q   Q ") ;
  sprintf(letters[16][2], "Q   Q ") ;
  sprintf(letters[16][3], " QQQ  ") ;
  sprintf(letters[16][4], "   QQ ") ;

  sprintf(letters[17][0], "RRRR  ") ;
  sprintf(letters[17][1], "R   R ") ;
  sprintf(letters[17][2], "RRRR  ") ;
  sprintf(letters[17][3], "R   R ") ;
  sprintf(letters[17][4], "R   R ") ;

  sprintf(letters[18][0], " SSSS ") ;
  sprintf(letters[18][1], "S     ") ;
  sprintf(letters[18][2], " SSS  ") ;
  sprintf(letters[18][3], "    S ") ;
  sprintf(letters[18][4], "SSSS  ") ;

  sprintf(letters[19][0], "TTTTT ") ;
  sprintf(letters[19][1], "  T   ") ;
  sprintf(letters[19][2], "  T   ") ;
  sprintf(letters[19][3], "  T   ") ;
  sprintf(letters[19][4], "  T   ") ;

  sprintf(letters[20][0], "U   U ") ;
  sprintf(letters[20][1], "U   U ") ;
  sprintf(letters[20][2], "U   U ") ;
  sprintf(letters[20][3], "U   U ") ;
  sprintf(letters[20][4], " UUU  ") ;

  sprintf(letters[21][0], "V   V ") ;
  sprintf(letters[21][1], "V   V ") ;
  sprintf(letters[21][2], "V   V ") ;
  sprintf(letters[21][3], " V V  ") ;
  sprintf(letters[21][4], "  V   ") ;

  sprintf(letters[22][0], "W   W ") ;
  sprintf(letters[22][1], "W   W ") ;
  sprintf(letters[22][2], "W W W ") ;
  sprintf(letters[22][3], "WW WW ") ;
  sprintf(letters[22][4], "W   W ") ;

  sprintf(letters[23][0], "X   X ") ;
  sprintf(letters[23][1], " X X  ") ;
  sprintf(letters[23][2], "  X   ") ;
  sprintf(letters[23][3], " X X  ") ;
  sprintf(letters[23][4], "X   X ") ;

  sprintf(letters[24][0], "Y   Y ") ;
  sprintf(letters[24][1], " Y Y  ") ;
  sprintf(letters[24][2], "  Y   ") ;
  sprintf(letters[24][3], "  Y   ") ;
  sprintf(letters[24][4], "  Y   ") ;

  sprintf(letters[25][0], "ZZZZZ ") ;
  sprintf(letters[25][1], "   Z  ") ;
  sprintf(letters[25][2], "  Z   ") ;
  sprintf(letters[25][3], " Z    ") ;
  sprintf(letters[25][4], "ZZZZZ ") ;

  sprintf(numbers[0][0], " 000  ") ;
  sprintf(numbers[0][1], "0  00 ") ;
  sprintf(numbers[0][2], "0 0 0 ") ;
  sprintf(numbers[0][3], "00  0 ") ;
  sprintf(numbers[0][4], " 000  ") ;

  sprintf(numbers[1][0], " 11   ") ;
  sprintf(numbers[1][1], "  1   ") ;
  sprintf(numbers[1][2], "  1   ") ;
  sprintf(numbers[1][3], "  1   ") ;
  sprintf(numbers[1][4], " 111  ") ;

  sprintf(numbers[2][0], " 222  ") ;
  sprintf(numbers[2][1], "2   2 ") ;
  sprintf(numbers[2][2], "   2  ") ;
  sprintf(numbers[2][3], " 22   ") ;
  sprintf(numbers[2][4], "22222 ") ;

  sprintf(numbers[3][0], " 333  ") ;
  sprintf(numbers[3][1], "3   3 ") ;
  sprintf(numbers[3][2], "   3  ") ;
  sprintf(numbers[3][3], "3   3 ") ;
  sprintf(numbers[3][4], " 333  ") ;

  sprintf(numbers[4][0], "4   4 ") ;
  sprintf(numbers[4][1], "4   4 ") ;
  sprintf(numbers[4][2], "44444 ") ;
  sprintf(numbers[4][3], "    4 ") ;
  sprintf(numbers[4][4], "    4 ") ;

  sprintf(numbers[5][0], "55555 ") ;
  sprintf(numbers[5][1], "5     ") ;
  sprintf(numbers[5][2], "5555  ") ;
  sprintf(numbers[5][3], "    5 ") ;
  sprintf(numbers[5][4], "5555  ") ;

  sprintf(numbers[6][0], " 666  ") ;
  sprintf(numbers[6][1], "6     ") ;
  sprintf(numbers[6][2], "6666  ") ;
  sprintf(numbers[6][3], "6   6 ") ;
  sprintf(numbers[6][4], "6666  ") ;

  sprintf(numbers[7][0], "77777 ") ;
  sprintf(numbers[7][1], "   7  ") ;
  sprintf(numbers[7][2], "  7   ") ;
  sprintf(numbers[7][3], " 7    ") ;
  sprintf(numbers[7][4], " 7    ") ;

  sprintf(numbers[8][0], " 888  ") ;
  sprintf(numbers[8][1], "8   8 ") ;
  sprintf(numbers[8][2], " 888  ") ;
  sprintf(numbers[8][3], "8   8 ") ;
  sprintf(numbers[8][4], " 888  ") ;

  sprintf(numbers[9][0], " 999  ") ;
  sprintf(numbers[9][1], "9   9 ") ;
  sprintf(numbers[9][2], " 9999 ") ;
  sprintf(numbers[9][3], "    9 ") ;
  sprintf(numbers[9][4], " 999  ") ;

  sprintf(punctuation[0][0], "   ") ;
  sprintf(punctuation[0][1], "OO ") ;
  sprintf(punctuation[0][2], "   ") ;
  sprintf(punctuation[0][3], "OO ") ;
  sprintf(punctuation[0][4], "   ") ;
}

/*
 * Send a line of text using complex looping technique
 *
 * NOTE: I couldn't figure out how to make this goofy
 * language recognize a plain vanilla NULL terminated
 * string so instead I am using the strlen function.
 * If it bothers you, fix it yourself. I am far too lazy.
 */ 
void vt100_send_line(char* theLine)
{
  int  thePos = 0 ;
  int  theLength = strlen(theLine) ;

  while (thePos != theLength)
  {
    bioscom(BIOSCOM_SEND, theLine[thePos++], COM_PORT) ;
  }
}


/*
 * Use vt100_send_line to print out large text using
 * advanced asterisk matrix technology.
 */
void font_print(char* theLine)
{
  int fontLine ;
  int thePos ;
  int theLength ;

  theLength = strlen(theLine) ;

    for (fontLine = 0  ;
         fontLine < 5 ;
         fontLine++)
    {
       for (thePos = 0 ;
            thePos < theLength ;
            thePos++)
       {
          if ((theLine[thePos] >= 'a') &&
             (theLine[thePos] <= 'z'))
            vt100_send_line(letters[theLine[thePos] - 97][fontLine]) ;
           else
          if ((theLine[thePos] >= 'A') &&
             (theLine[thePos] <= 'Z'))
            vt100_send_line(letters[theLine[thePos] - 65][fontLine]) ;
           else
          if ((theLine[thePos] >= '0') &&
             (theLine[thePos] <= '9'))
            vt100_send_line(numbers[theLine[thePos] - 48][fontLine]) ;
          else
            if (theLine[thePos] == ':')
              vt100_send_line(punctuation[0][fontLine]) ;
            else
              vt100_send_line("      ") ;
       }

       vt100_send_line("\n\r") ;
    }

    vt100_send_line("\n\r") ;
}

/*===========================================================================
  void lcd_driver_init()

  This function should set-up the LCD, clear it, and return the cursor
  to the home position.
  =========================================================================*/

void lcd_driver_init()
{
   /* First we put our driver details into the appropriate variables */
   sprintf(lcd_driver_name,"vt100 serial driver v1.0") ;
   sprintf(lcd_driver_author,"Adam Briggs") ;
   sprintf(lcd_driver_author_email,"atom_bomb@rocketmail.com") ;
   sprintf(lcd_driver_description,"Output on any vt100 compatible device") ;
   lcd_driver_version = 1.0 ;

   /* Setup what characters we'd like to use for the various symbols */
   lcd_surround_symbol = 'O' ;
   lcd_play_symbol = '>' ;
   lcd_random_symbol = '?' ;
   lcd_rew_symbol = '<' ;
   lcd_continuous_symbol = 'C' ;

   /* Now we'll initialise the display */
   bioscom(BIOSCOM_INIT, COM_INIT, COM_PORT) ;
   bioscom(BIOSCOM_SEND, VT100_ESCAPE, COM_PORT) ;
   vt100_send_line(VT100_INIT) ;
   bioscom(BIOSCOM_SEND, VT100_ESCAPE, COM_PORT) ;
   vt100_send_line(VT100_CLEAR) ;
   bioscom(BIOSCOM_SEND, VT100_ESCAPE, COM_PORT) ;
   vt100_send_line(VT100_HOME) ;
   
   /* can you guess what this initializes? */
   init_font() ;
}

/*===========================================================================
  void lcd_driver_clear()

  This function should clear the LCD display.
  =========================================================================*/

void lcd_driver_clear()
{
   bioscom(BIOSCOM_SEND, VT100_ESCAPE, COM_PORT) ;
   vt100_send_line(VT100_CLEAR) ;
}


/*===========================================================================
  void lcd_driver_home()

  This function should return the LCD cursor to the home position.
  =========================================================================*/

void lcd_driver_home()
{
   bioscom(BIOSCOM_SEND, VT100_ESCAPE, COM_PORT) ;
   vt100_send_line(VT100_HOME) ;
}


/*==========================================================================
  void lcd_driver_printf()

  This function should take the two strings lcd_line[0] and lcd_line[1] and
  output them onto the top and bottom lines of the LCD.
  ========================================================================*/

int  my_track_number ;
char last_digit ;

void lcd_driver_printf()
{
  int textLine ;
  char trackNumberString[80] ;

  /*
   * Output the typical DAMP lcd output
   * at the top of the screen.
   */
  for (textLine = 0 ;
       textLine < 2 ;
       textLine++)
  {
    vt100_send_line(lcd_line[textLine]) ;
    vt100_send_line("\n\r") ;
  }

  vt100_send_line("\n\r") ;

  /*
   * Only output the time remaining when it changes
   * since it is possible that this function will be
   * called more than once per second.
   */
  if ((damp_time_remaining[4] != last_digit) ||
     (damp_track_number != my_track_number))
  {
    last_digit = damp_time_remaining[4] ;
    font_print(damp_time_remaining) ;   
  }

  /*
   * Only update the id3 output when the user
   * has changed to a new track.
   */
  if (damp_track_number != my_track_number)
  {
    /*
     * use some vt100 magic to clear the old id3
     * strings without wiping out what has already
     * been written to the top of the screen.
     */
    bioscom(BIOSCOM_SEND, VT100_ESCAPE, COM_PORT) ;
    vt100_send_line(VT100_CLEARTOEOS) ;

    sprintf(trackNumberString, "%d", damp_track_number) ;
    font_print(trackNumberString) ;
    my_track_number = damp_track_number ;

    vt100_send_line(damp_id3_title) ;
    vt100_send_line("\n\r") ;
    vt100_send_line(damp_id3_artist) ;
    vt100_send_line("\n\r") ;
    vt100_send_line(damp_id3_album) ;
    vt100_send_line("\n\r") ;
    vt100_send_line(damp_id3_year) ;
    vt100_send_line("\n\r") ;
    vt100_send_line(damp_id3_comment) ;
    vt100_send_line("\n\r") ;
  }
}
